@@ -8,7 +8,7 @@ package Device::BusPirate;
 use strict;
 use warnings;
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 use Carp;
 
@@ -160,7 +160,7 @@ sub read
 
    return Future->wait_any(
       $f,
-      $self->sleep( $timeout // 0.5 )->then_fail( "Timeout waiting for $name" ),
+      $self->sleep( $timeout // 2 )->then_fail( "Timeout waiting for $name" ),
    );
 }
 
