@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Device::BusPirate::Mode::BB;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Device::BusPirate::Mode );
 
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 
 use Carp;
 
@@ -135,11 +135,11 @@ configuration setting to determine whether high should be hi-Z or 3.3V.
 
 =cut
 
-sub cs   { shift->_write( 0, cs   => $_[0] ) }
-sub miso { shift->_write( 0, miso => $_[0] ) }
-sub clk  { shift->_write( 0, clk  => $_[0] ) }
-sub mosi { shift->_write( 0, mosi => $_[0] ) }
-sub aux  { shift->_write( 0, aux  => $_[0] ) }
+sub cs   { shift->_writeread( 0, { cs   => $_[0] }, [] ) }
+sub miso { shift->_writeread( 0, { miso => $_[0] }, [] ) }
+sub clk  { shift->_writeread( 0, { clk  => $_[0] }, [] ) }
+sub mosi { shift->_writeread( 0, { mosi => $_[0] }, [] ) }
+sub aux  { shift->_writeread( 0, { aux  => $_[0] }, [] ) }
 
 =head2 write
 
@@ -149,23 +149,24 @@ Sets the state of multiple output pins at the same time.
 
 =cut
 
-sub _write
+sub _writeread
 {
    my $self = shift;
-   my ( $want_read, %pins ) = @_;
+   my ( $want_read, $pins_write, $pins_read ) = @_;
 
    my $out = $self->{out_mask};
    my $dir = $self->{dir_mask};
 
-   foreach my $pin ( keys %PIN_MASK ) {
-      next unless exists $pins{$pin};
-      my $mask = $PIN_MASK{$pin};
+   foreach my $pin ( keys %$pins_write ) {
+      my $mask = $PIN_MASK{$pin} or
+         croak "Unrecognised BB pin name $pin";
+      my $val = $pins_write->{$pin};
 
-      if( $pins{$pin} and !$self->{open_drain} ) {
+      if( $val and !$self->{open_drain} ) {
          $dir &= ~$mask;
          $out |=  $mask;
       }
-      elsif( $pins{$pin} ) {
+      elsif( $val ) {
          $dir |=  $mask;
       }
       else {
@@ -174,6 +175,13 @@ sub _write
       }
    }
 
+   foreach my $pin ( @$pins_read ) {
+      my $mask = $PIN_MASK{$pin} or
+         croak "Unrecognised BB pin name $pin";
+
+      $dir |= $mask;
+   }
+
    my $len = 0;
    if( $dir != $self->{dir_mask} ) {
       $self->pirate->write( chr( 0x40 | $dir ) );
@@ -192,14 +200,26 @@ sub _write
    return Future->done unless $len;
 
    my $f = $self->pirate->read( $len );
-   return $f if $want_read;
-   return $f->then_done(); # ignore input
+
+   return $f->then_done() if !$want_read;
+   return $f->then( sub {
+      my ( $buf ) = @_;
+      $buf = ord $buf;
+
+      my $pins;
+      foreach my $pin ( keys %PIN_MASK ) {
+         my $mask = $PIN_MASK{$pin};
+         next unless $self->{dir_mask} & $mask;
+         $pins->{$pin} = !!( $buf & $mask );
+      }
+      Future->done( $pins );
+   });
 }
 
 sub write
 {
    my $self = shift;
-   $self->_write( 0, @_ );
+   $self->_writeread( 0, { @_ }, [] );
 }
 
 sub read_cs   { shift->_input1( MASK_CS,   @_ ) }
@@ -255,18 +275,19 @@ sub _input
 
 =head2 read
 
-   $pins = $bbio->read->get
+   $pins = $bbio->read( @pins )->get
 
-Returns a HASH containing the current state of all the pins currently
-configured as inputs. More efficient than calling multiple C<read_*> methods
-when more than one pin is being read at the same time.
+Sets given list of pins (which may be empty) to be inputs, and returns a HASH
+containing the current state of all the pins currently configured as inputs.
+More efficient than calling multiple C<read_*> methods when more than one pin
+is being read at the same time.
 
 =cut
 
 sub read
 {
    my $self = shift;
-   $self->writeread();
+   $self->_writeread( 1, {}, [ @_ ] );
 }
 
 =head2 writeread
@@ -282,18 +303,7 @@ pins currently set as inputs.
 sub writeread
 {
    my $self = shift;
-   $self->_write( 1, @_ )->then( sub {
-      my ( $buf ) = @_;
-      $buf = ord $buf;
-
-      my $pins;
-      foreach my $pin ( keys %PIN_MASK ) {
-         my $mask = $PIN_MASK{$pin};
-         next unless $self->{dir_mask} & $mask;
-         $pins->{$pin} = !!( $buf & $mask );
-      }
-      Future->done( $pins );
-   });
+   $self->_writeread( 1, { @_ }, [] );
 }
 
 =head2 power
@@ -335,6 +345,17 @@ sub pullup
    $self->pirate->read( 1 )->then_done(); # ignore input
 }
 
+=head1 TODO
+
+=over 4
+
+=item *
+
+Some method of setting multiple pins into read mode at once, so that a single
+C<read> method hits them all.
+
+=back
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
