@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2015-2016 -- leonerd@leonerd.org.uk
 
 package Device::Chip::Adapter::BusPirate;
 
@@ -63,6 +63,21 @@ L<Device::Chip::Adapter/METHODS> at version 0.01.
 
 =cut
 
+sub make_protocol_GPIO
+{
+   my $self = shift;
+
+   $self->{bp}->enter_mode( "BB" )->then( sub {
+      my ( $mode ) = @_;
+      $self->{mode} = $mode;
+
+      $mode->configure( open_drain => 0 )
+         ->then_done(
+            Device::Chip::Adapter::BusPirate::_GPIO->new( $mode )
+         );
+   });
+}
+
 sub make_protocol_SPI
 {
    my $self = shift;
@@ -103,6 +118,7 @@ sub shutdown
 package
    Device::Chip::Adapter::BusPirate::_base;
 
+use Carp;
 use List::Util qw( first );
 
 sub new
@@ -113,6 +129,12 @@ sub new
    bless { mode => $mode }, $class;
 }
 
+sub power
+{
+   my $self = shift;
+   $self->{mode}->power( @_ );
+}
+
 sub _find_speed
 {
    shift;
@@ -127,6 +149,86 @@ sub _find_speed
     } @speeds;
 }
 
+# Most modes only have access to the AUX GPIO pin
+sub list_gpios { return qw( AUX ) }
+
+sub write_gpios
+{
+   my $self = shift;
+   my ( $gpios ) = @_;
+
+   my $mode = $self->{mode};
+
+   foreach my $pin ( keys %$gpios ) {
+      $pin eq "AUX" or
+         croak "Unrecognised GPIO pin name $pin";
+
+      return $mode->aux( $gpios->{$pin} );
+   }
+
+   Future->done;
+}
+
+sub read_gpios
+{
+   my $self = shift;
+   my ( $gpios ) = @_;
+
+   my $mode = $self->{mode};
+
+   my @f;
+   foreach my $pin ( @$gpios ) {
+      $pin eq "AUX" or
+         croak "Unrecognised GPIO pin name $pin";
+
+      return $mode->read_aux
+         ->transform( done => sub { { AUX => $_[0] } } );
+   }
+
+   Future->done( {} );
+}
+
+# there's no more efficient way to tris_gpios than just read and ignore the result
+sub tris_gpios
+{
+   my $self = shift;
+   $self->read_gpios->then_done();
+}
+
+package
+   Device::Chip::Adapter::BusPirate::_GPIO;
+use base qw( Device::Chip::Adapter::BusPirate::_base );
+
+use List::Util 1.29 qw( pairmap );
+
+sub list_gpios { return qw( MISO CS MOSI CLK AUX ) }
+
+sub write_gpios
+{
+   my $self = shift;
+   my ( $gpios ) = @_;
+
+   my $mode = $self->{mode};
+
+   # TODO: validity checking
+   $mode->write(
+      pairmap { lc $a => $b } %$gpios
+   )
+}
+
+sub read_gpios
+{
+   my $self = shift;
+   my ( $gpios ) = @_;
+
+   my $mode = $self->{mode};
+
+   $mode->read( map { lc $_ } @$gpios )->then( sub {
+      my ( $vals ) = @_;
+      Future->done( { pairmap { uc $a => $b } %$vals } );
+   });
+}
+
 package
    Device::Chip::Adapter::BusPirate::_SPI;
 use base qw( Device::Chip::Adapter::BusPirate::_base );
@@ -154,12 +256,6 @@ sub configure
     );
 }
 
-sub power
-{
-   my $self = shift;
-   $self->{mode}->power( @_ );
-}
-
 sub readwrite
 {
    my $self = shift;
@@ -180,17 +276,10 @@ sub write
 
 package
     Device::Chip::Adapter::BusPirate::_I2C;
+use base qw( Device::Chip::Adapter::BusPirate::_base );
 
 use Carp;
 
-sub new
-{
-    my $class = shift;
-    my ( $mode ) = @_;
-
-    bless { mode => $mode }, $class;
-}
-
 my @I2C_SPEEDS = (qw( 400k 100k 50k 5k ));
 
 # TODO - addr ought to be a mount option somehow
@@ -216,12 +305,6 @@ sub configure
     Future->needs_all( @f );
 }
 
-sub power
-{
-   my $self = shift;
-   $self->{mode}->power( @_ );
-}
-
 sub write
 {
     my $self = shift;
@@ -234,6 +317,12 @@ sub write_then_read
     $self->{mode}->send_then_recv( $self->{addr}, @_ );
 }
 
+sub read
+{
+   my $self = shift;
+   $self->{mode}->recv( $self->{addr}, @_ );
+}
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
